import time
import unittest

import pkg_resources
from mock_decorators.function_mock import FunctionMock, FunctionMockResult

from git_cloner.model.owner.github_owner import GithubOwner
from git_cloner.model.remotes.github_remote import GithubRemote
from git_cloner.model.repository import Repository
from git_cloner.operations.mass_cloner import *
from git_cloner.utils import utils
from git_cloner.utils.config import Config
from tests.git_cloner.utils.gen_utils import generate_repo_github
from tests.git_cloner.utils.utils import *

test_dir = "/tmp/test_tmp"
response_filepath = pkg_resources.resource_filename("tests", "resources/request_fhuertas_org.json")
log_retry_path = pkg_resources.resource_filename("tests", "resources/retry.log")


class MassClonerIT(unittest.TestCase):
    def setUp(self):
        self.dir = set_test_dir(test_dir)
        time.sleep(1)

    def tearDown(self):
        restore_path(self.dir)
        time.sleep(1)

    def test_serial_it(self):
        def test_mass_cloner_user():
            config = Config.load_config({})
            config.type = "github"
            config.target_user = "yunxao"
            config.dir_out = test_dir
            mass_cloner()
            repos = [name for name in os.listdir(test_dir) if Repository(os.path.join(test_dir, name)).check_git()]
            self.assertEqual(len(repos), 10)

        def test_mass_cloner_org():
            config = Config.load_config({})
            config.type = "github"
            config.org = "fhuertasorgfortest"
            config.ssh = True
            config.dir_out = test_dir
            mass_cloner()
            repos = [name for name in os.listdir(test_dir) if Repository(os.path.join(test_dir, name)).check_git()]
            self.assertEqual(len(repos), 1)

        def test_mass_cloner_fail():
            config = Config.load_config({})
            config.type = "github"
            config.org = "fhuertasorgfortest"
            config.ssh = True
            config.dir_out = test_dir

            def mock_request(*args, **kwargs):
                result = json.loads(utils.read(response_filepath))
                return result

            @FunctionMock(utils, 'make_request', mock_request, False)
            def inner_test():
                mass_cloner()

            inner_test()
            repos = [name for name in os.listdir(test_dir) if Repository(os.path.join(test_dir, name)).check_git()]
            self.assertEqual(len(repos), 1)

        def test_logger():
            config = Config.load_config({})
            config.type = Repository.GITHUB
            config.org = "fhuertasorgfortest"
            config.ssh = True
            config.save = os.path.join(test_dir, "log.log")
            config.dir_out = test_dir

            repos_raw = [
                generate_repo_github(name="monkey", user="fhuertasorgfortest"),
                generate_repo_github()]
            repos = [RawRepository.from_raw_data(repo, config.type) for repo in repos_raw]

            @FunctionMockResult(entity=GithubOwner, function_name="get_repos", result=repos)
            def inner_test():
                mass_cloner()

            inner_test()

            with open(config.save) as file:
                result = "".join(file.readlines()).strip()
                self.assertEqual(result,
                                 "Server: github, Repo: fhuertasorgfortest/monkey, Operation: clone, Result: True\n"
                                 "Server: github, Repo: {}, Operation: clone, Result: False".format(
                                     repos_raw[1].get('full_name')))

        def test_get_repos_normal():
            config = Config.load_config({})
            config.type = "github"
            config.org = "fhuertasorgfortest"
            config.ssh = True
            result = get_repos()
            self.assertEqual(len(result), 1)

        def test_get_repos_retry():
            config = Config.load_config({})
            config.type = "github"
            config.org = "fhuertasorgfortest"
            config.ssh = True
            config.retry = log_retry_path
            result = get_repos()
            self.assertEqual(len(result), 2)

            self.assertTrue(isinstance(result[0].remote, GithubRemote))
            self.assertEqual(result[0].remote.repo_name, 'monkey')
            self.assertEqual(result[0].remote.owner, 'fhuertasorgfortest')
            self.assertTrue(isinstance(result[1].remote, GithubRemote))
            self.assertEqual(result[1].remote.repo_name, 'mix')
            self.assertEqual(result[1].remote.owner, 'mix')

        def test_retry_mass_clone():
            config = Config.load_config({})
            config.type = "github"
            config.org = "fhuertasorgfortest"
            config.ssh = True
            config.retry = log_retry_path
            result = mass_cloner()

            self.assertEqual(len(result), 1)

        self.setUp()
        test_mass_cloner_user()
        self.tearDown()
        self.setUp()
        test_mass_cloner_org()
        self.tearDown()
        self.setUp()
        test_mass_cloner_fail()
        self.tearDown()
        self.setUp()
        test_logger()
        test_get_repos_normal()
        self.setUp()
        test_get_repos_retry()
        self.tearDown()
        self.setUp()
        test_retry_mass_clone()
